#ifndef _PYZMQ_CONSTANT_DEFS
#define _PYZMQ_CONSTANT_DEFS

#ifndef ZMQ_VERSION
    #define ZMQ_VERSION (-1)
#endif

#ifndef ZMQ_VERSION_MAJOR
    #define ZMQ_VERSION_MAJOR (-1)
#endif

#ifndef ZMQ_VERSION_MINOR
    #define ZMQ_VERSION_MINOR (-1)
#endif

#ifndef ZMQ_VERSION_PATCH
    #define ZMQ_VERSION_PATCH (-1)
#endif

#ifndef ZMQ_NOBLOCK
    #define ZMQ_NOBLOCK (-1)
#endif

#ifndef ZMQ_DONTWAIT
    #define ZMQ_DONTWAIT (-1)
#endif

#ifndef ZMQ_POLLIN
    #define ZMQ_POLLIN (-1)
#endif

#ifndef ZMQ_POLLOUT
    #define ZMQ_POLLOUT (-1)
#endif

#ifndef ZMQ_POLLERR
    #define ZMQ_POLLERR (-1)
#endif

#ifndef ZMQ_SNDMORE
    #define ZMQ_SNDMORE (-1)
#endif

#ifndef ZMQ_STREAMER
    #define ZMQ_STREAMER (-1)
#endif

#ifndef ZMQ_FORWARDER
    #define ZMQ_FORWARDER (-1)
#endif

#ifndef ZMQ_QUEUE
    #define ZMQ_QUEUE (-1)
#endif

#ifndef ZMQ_IO_THREADS_DFLT
    #define ZMQ_IO_THREADS_DFLT (-1)
#endif

#ifndef ZMQ_MAX_SOCKETS_DFLT
    #define ZMQ_MAX_SOCKETS_DFLT (-1)
#endif

#ifndef ZMQ_PAIR
    #define ZMQ_PAIR (-1)
#endif

#ifndef ZMQ_PUB
    #define ZMQ_PUB (-1)
#endif

#ifndef ZMQ_SUB
    #define ZMQ_SUB (-1)
#endif

#ifndef ZMQ_REQ
    #define ZMQ_REQ (-1)
#endif

#ifndef ZMQ_REP
    #define ZMQ_REP (-1)
#endif

#ifndef ZMQ_DEALER
    #define ZMQ_DEALER (-1)
#endif

#ifndef ZMQ_ROUTER
    #define ZMQ_ROUTER (-1)
#endif

#ifndef ZMQ_PULL
    #define ZMQ_PULL (-1)
#endif

#ifndef ZMQ_PUSH
    #define ZMQ_PUSH (-1)
#endif

#ifndef ZMQ_XPUB
    #define ZMQ_XPUB (-1)
#endif

#ifndef ZMQ_XSUB
    #define ZMQ_XSUB (-1)
#endif

#ifndef ZMQ_UPSTREAM
    #define ZMQ_UPSTREAM (-1)
#endif

#ifndef ZMQ_DOWNSTREAM
    #define ZMQ_DOWNSTREAM (-1)
#endif

#ifndef ZMQ_STREAM
    #define ZMQ_STREAM (-1)
#endif

#ifndef ZMQ_EVENT_CONNECTED
    #define ZMQ_EVENT_CONNECTED (-1)
#endif

#ifndef ZMQ_EVENT_CONNECT_DELAYED
    #define ZMQ_EVENT_CONNECT_DELAYED (-1)
#endif

#ifndef ZMQ_EVENT_CONNECT_RETRIED
    #define ZMQ_EVENT_CONNECT_RETRIED (-1)
#endif

#ifndef ZMQ_EVENT_LISTENING
    #define ZMQ_EVENT_LISTENING (-1)
#endif

#ifndef ZMQ_EVENT_BIND_FAILED
    #define ZMQ_EVENT_BIND_FAILED (-1)
#endif

#ifndef ZMQ_EVENT_ACCEPTED
    #define ZMQ_EVENT_ACCEPTED (-1)
#endif

#ifndef ZMQ_EVENT_ACCEPT_FAILED
    #define ZMQ_EVENT_ACCEPT_FAILED (-1)
#endif

#ifndef ZMQ_EVENT_CLOSED
    #define ZMQ_EVENT_CLOSED (-1)
#endif

#ifndef ZMQ_EVENT_CLOSE_FAILED
    #define ZMQ_EVENT_CLOSE_FAILED (-1)
#endif

#ifndef ZMQ_EVENT_DISCONNECTED
    #define ZMQ_EVENT_DISCONNECTED (-1)
#endif

#ifndef ZMQ_EVENT_ALL
    #define ZMQ_EVENT_ALL (-1)
#endif

#ifndef ZMQ_EVENT_MONITOR_STOPPED
    #define ZMQ_EVENT_MONITOR_STOPPED (-1)
#endif

#ifndef ZMQ_NULL
    #define ZMQ_NULL (-1)
#endif

#ifndef ZMQ_PLAIN
    #define ZMQ_PLAIN (-1)
#endif

#ifndef ZMQ_CURVE
    #define ZMQ_CURVE (-1)
#endif

#ifndef EAGAIN
    #define EAGAIN (-1)
#endif

#ifndef EINVAL
    #define EINVAL (-1)
#endif

#ifndef EFAULT
    #define EFAULT (-1)
#endif

#ifndef ENOMEM
    #define ENOMEM (-1)
#endif

#ifndef ENODEV
    #define ENODEV (-1)
#endif

#ifndef EMSGSIZE
    #define EMSGSIZE (-1)
#endif

#ifndef EAFNOSUPPORT
    #define EAFNOSUPPORT (-1)
#endif

#ifndef ENETUNREACH
    #define ENETUNREACH (-1)
#endif

#ifndef ECONNABORTED
    #define ECONNABORTED (-1)
#endif

#ifndef ECONNRESET
    #define ECONNRESET (-1)
#endif

#ifndef ENOTCONN
    #define ENOTCONN (-1)
#endif

#ifndef ETIMEDOUT
    #define ETIMEDOUT (-1)
#endif

#ifndef EHOSTUNREACH
    #define EHOSTUNREACH (-1)
#endif

#ifndef ENETRESET
    #define ENETRESET (-1)
#endif

#ifndef ZMQ_HAUSNUMERO
    #define ZMQ_HAUSNUMERO (-1)
#endif

#ifndef ENOTSUP
    #define ENOTSUP (-1)
#endif

#ifndef EPROTONOSUPPORT
    #define EPROTONOSUPPORT (-1)
#endif

#ifndef ENOBUFS
    #define ENOBUFS (-1)
#endif

#ifndef ENETDOWN
    #define ENETDOWN (-1)
#endif

#ifndef EADDRINUSE
    #define EADDRINUSE (-1)
#endif

#ifndef EADDRNOTAVAIL
    #define EADDRNOTAVAIL (-1)
#endif

#ifndef ECONNREFUSED
    #define ECONNREFUSED (-1)
#endif

#ifndef EINPROGRESS
    #define EINPROGRESS (-1)
#endif

#ifndef ENOTSOCK
    #define ENOTSOCK (-1)
#endif

#ifndef EFSM
    #define EFSM (-1)
#endif

#ifndef ENOCOMPATPROTO
    #define ENOCOMPATPROTO (-1)
#endif

#ifndef ETERM
    #define ETERM (-1)
#endif

#ifndef EMTHREAD
    #define EMTHREAD (-1)
#endif

#ifndef ZMQ_IO_THREADS
    #define ZMQ_IO_THREADS (-1)
#endif

#ifndef ZMQ_MAX_SOCKETS
    #define ZMQ_MAX_SOCKETS (-1)
#endif

#ifndef ZMQ_MORE
    #define ZMQ_MORE (-1)
#endif

#ifndef ZMQ_IDENTITY
    #define ZMQ_IDENTITY (-1)
#endif

#ifndef ZMQ_SUBSCRIBE
    #define ZMQ_SUBSCRIBE (-1)
#endif

#ifndef ZMQ_UNSUBSCRIBE
    #define ZMQ_UNSUBSCRIBE (-1)
#endif

#ifndef ZMQ_LAST_ENDPOINT
    #define ZMQ_LAST_ENDPOINT (-1)
#endif

#ifndef ZMQ_TCP_ACCEPT_FILTER
    #define ZMQ_TCP_ACCEPT_FILTER (-1)
#endif

#ifndef ZMQ_PLAIN_USERNAME
    #define ZMQ_PLAIN_USERNAME (-1)
#endif

#ifndef ZMQ_PLAIN_PASSWORD
    #define ZMQ_PLAIN_PASSWORD (-1)
#endif

#ifndef ZMQ_CURVE_PUBLICKEY
    #define ZMQ_CURVE_PUBLICKEY (-1)
#endif

#ifndef ZMQ_CURVE_SECRETKEY
    #define ZMQ_CURVE_SECRETKEY (-1)
#endif

#ifndef ZMQ_CURVE_SERVERKEY
    #define ZMQ_CURVE_SERVERKEY (-1)
#endif

#ifndef ZMQ_ZAP_DOMAIN
    #define ZMQ_ZAP_DOMAIN (-1)
#endif

#ifndef ZMQ_RECONNECT_IVL_MAX
    #define ZMQ_RECONNECT_IVL_MAX (-1)
#endif

#ifndef ZMQ_SNDTIMEO
    #define ZMQ_SNDTIMEO (-1)
#endif

#ifndef ZMQ_RCVTIMEO
    #define ZMQ_RCVTIMEO (-1)
#endif

#ifndef ZMQ_SNDHWM
    #define ZMQ_SNDHWM (-1)
#endif

#ifndef ZMQ_RCVHWM
    #define ZMQ_RCVHWM (-1)
#endif

#ifndef ZMQ_MULTICAST_HOPS
    #define ZMQ_MULTICAST_HOPS (-1)
#endif

#ifndef ZMQ_IPV4ONLY
    #define ZMQ_IPV4ONLY (-1)
#endif

#ifndef ZMQ_ROUTER_BEHAVIOR
    #define ZMQ_ROUTER_BEHAVIOR (-1)
#endif

#ifndef ZMQ_TCP_KEEPALIVE
    #define ZMQ_TCP_KEEPALIVE (-1)
#endif

#ifndef ZMQ_TCP_KEEPALIVE_CNT
    #define ZMQ_TCP_KEEPALIVE_CNT (-1)
#endif

#ifndef ZMQ_TCP_KEEPALIVE_IDLE
    #define ZMQ_TCP_KEEPALIVE_IDLE (-1)
#endif

#ifndef ZMQ_TCP_KEEPALIVE_INTVL
    #define ZMQ_TCP_KEEPALIVE_INTVL (-1)
#endif

#ifndef ZMQ_DELAY_ATTACH_ON_CONNECT
    #define ZMQ_DELAY_ATTACH_ON_CONNECT (-1)
#endif

#ifndef ZMQ_XPUB_VERBOSE
    #define ZMQ_XPUB_VERBOSE (-1)
#endif

#ifndef ZMQ_FD
    #define ZMQ_FD (-1)
#endif

#ifndef ZMQ_EVENTS
    #define ZMQ_EVENTS (-1)
#endif

#ifndef ZMQ_TYPE
    #define ZMQ_TYPE (-1)
#endif

#ifndef ZMQ_LINGER
    #define ZMQ_LINGER (-1)
#endif

#ifndef ZMQ_RECONNECT_IVL
    #define ZMQ_RECONNECT_IVL (-1)
#endif

#ifndef ZMQ_BACKLOG
    #define ZMQ_BACKLOG (-1)
#endif

#ifndef ZMQ_ROUTER_MANDATORY
    #define ZMQ_ROUTER_MANDATORY (-1)
#endif

#ifndef ZMQ_FAIL_UNROUTABLE
    #define ZMQ_FAIL_UNROUTABLE (-1)
#endif

#ifndef ZMQ_ROUTER_RAW
    #define ZMQ_ROUTER_RAW (-1)
#endif

#ifndef ZMQ_IMMEDIATE
    #define ZMQ_IMMEDIATE (-1)
#endif

#ifndef ZMQ_IPV6
    #define ZMQ_IPV6 (-1)
#endif

#ifndef ZMQ_MECHANISM
    #define ZMQ_MECHANISM (-1)
#endif

#ifndef ZMQ_PLAIN_SERVER
    #define ZMQ_PLAIN_SERVER (-1)
#endif

#ifndef ZMQ_CURVE_SERVER
    #define ZMQ_CURVE_SERVER (-1)
#endif

#ifndef ZMQ_PROBE_ROUTER
    #define ZMQ_PROBE_ROUTER (-1)
#endif

#ifndef ZMQ_REQ_RELAXED
    #define ZMQ_REQ_RELAXED (-1)
#endif

#ifndef ZMQ_REQ_CORRELATE
    #define ZMQ_REQ_CORRELATE (-1)
#endif

#ifndef ZMQ_CONFLATE
    #define ZMQ_CONFLATE (-1)
#endif

#ifndef ZMQ_AFFINITY
    #define ZMQ_AFFINITY (-1)
#endif

#ifndef ZMQ_MAXMSGSIZE
    #define ZMQ_MAXMSGSIZE (-1)
#endif

#ifndef ZMQ_HWM
    #define ZMQ_HWM (-1)
#endif

#ifndef ZMQ_SWAP
    #define ZMQ_SWAP (-1)
#endif

#ifndef ZMQ_MCAST_LOOP
    #define ZMQ_MCAST_LOOP (-1)
#endif

#ifndef ZMQ_RECOVERY_IVL_MSEC
    #define ZMQ_RECOVERY_IVL_MSEC (-1)
#endif

#ifndef ZMQ_RATE
    #define ZMQ_RATE (-1)
#endif

#ifndef ZMQ_RECOVERY_IVL
    #define ZMQ_RECOVERY_IVL (-1)
#endif

#ifndef ZMQ_SNDBUF
    #define ZMQ_SNDBUF (-1)
#endif

#ifndef ZMQ_RCVBUF
    #define ZMQ_RCVBUF (-1)
#endif

#ifndef ZMQ_RCVMORE
    #define ZMQ_RCVMORE (-1)
#endif


#endif // ifndef _PYZMQ_CONSTANT_DEFS
